#!/usr/bin/python
# -*- coding: ascii -*-
# $Id$
#
# Author: Mario Basz
#mariob_1960@yahoo.com.ar
# Date: 9 November 2017
# Date: 03 may 2018
# A special thanks to Filippo Rivato and Vasilis.
#This plugin is based on a variation
# of yours plugin Driller and My_Plugin example.
# To correct: Thats why the first point starts,

from __future__ import absolute_import
from __future__ import print_function
__author__ = "Mario Basz"
__email__  = "mariob_1960@yahoo.com.ar"

__name__ = _("Trochoidal Path")
__version__= "1.0"
# Date last version: 29-January-2019

import math
from bmath import Vector
#import CNC					 # <<
from CNC import CNC,Block   #<< without this error it does not find CNC.vars
from ToolsPage import Plugin


#==============================================================================
# Create Trochoidadl rute along selected blocks
#==============================================================================
class Tool(Plugin):
	__doc__ = _("Create a trochoid rute along selected blocks")

	def __init__(self, master):
		Plugin.__init__(self, master, "Trochoid_Path") #NAME OF THE PLUGIN
		self.icon  = "trochoidal"
		self.group = "Development"

		self.variables = [
			("name",      "db" ,                     "", _("Name")),
	        ("trochcutdiam" ,"mm",                  6.0, _("Cut Diameter"),"Real cutting diameter"),
			("direction","inside,outside,on" , "inside", _("Direction")),
			("offset",   "float",                   0.0, _("Additional offset distance")),
			("endmill",   "db" ,                     "", _("End Mill"), "If Empty chooses, End Mill loaded"),
			("adaptative",  "bool",                   1,   _("Adaptative"), "Generate path for adaptative trochoids in the corners (Not yet implemented in trochoidal plugin)"),
			("overcut",  "bool",                      0, _("Overcut")),
			("targetDepth",  "mm",                    -1, _("Target Depth")),
#			("depthIncrement",  "mm",                  1, _("Depth Increment")),
			("depthIncrement",  "mm",                  1, _("Depth Increment")),
			("tabsnumber",  "mm",                      1, _("Number of Tabs 0 = Not Tabs"),"Not available yet"),
			("tabsWidth",  "mm",                       1, _("Tabs Diameter"),"Not available yet"),
			("tabsHeight",  "mm",                      1, _("Tabs Height"),"Not available yet"),

#			("mintrochdiam", "float",                10, _("Minimal trochoid in % tool"))
		]
		self.buttons.append("exe")

	# ----------------------------------------------------------------------
	def update(self):
		self.master.cnc()["trochcutdiam"] = self.fromMm("trochcutdiam")
	# ----------------------------------------------------------------------
	def execute(self, app):
		if self["endmill"]:
			self.master["endmill"].makeCurrent(self["endmill"])
			
		targetDepth=self["targetDepth"]
#		if targetDepth=="":
#			targetDepth=CNC.vars["surface"]
		depthIncrement=self["depthIncrement"]
		if depthIncrement=="":
			depthIncrement=10
		tabsnumber=self["tabsnumber"]
		tabsWidth=self["tabsWidth"]
		tabsHeight=self["tabsHeight"]
			
		trochcutdiam=self.fromMm("trochcutdiam")
#		mintrochdiameter = CNC.vars["diameter"]*(1+self["mintrochdiam"]/100.0)
		mintrochdiameter = CNC.vars["diameter"]
		cornerradius = (trochcutdiam - mintrochdiameter) /2.0
		direction = self["direction"]
		name = self["name"]
		if name=="default" or name=="": name=None
		app.trochprofile_bcnc(trochcutdiam, direction, self["offset"], self["overcut"], self["adaptative"], cornerradius, CNC.vars["diameter"],\
			targetDepth, depthIncrement, tabsnumber, tabsWidth, tabsHeight) #<< diameter only to information
		app.setStatus(_("Generated path for trochoidal cutting"))

#==============================================================================



#!/usr/bin/python
# -*- coding: ascii -*-
# $Id$
#
# Author:	DodoLaSaumure
# Date:	30-Dec-2019

from __future__ import absolute_import
from __future__ import print_function
__author__ = "DodoLaSaumure"
__email__  = ""


import math
from bmath import Vector
from CNC import CW,CNC,Block
from ToolsPage import Plugin
from CNCRibbon    import Page
try:
# 	import Tkinter
# 	from Tkinter import *
	import tkMessageBox
except ImportError:
# 	import tkinter
# 	from tkinter import *
	import tkinter.messagebox as tkMessageBox


#==============================================================================
# Create a simple Rotate
#==============================================================================
class Tool(Plugin):
	__doc__ = _("Rotates a block to a new position")

	def __init__(self, master):
		Plugin.__init__(self, master, "SimpleRotate")
		self.icon  = "SimpleRotate"
		self.group = "Generator"
		self.variables = [
			("xcenter",        "float",    10.0, _("x center")),
			("ycenter",        "float",    10.0, _("y center")),
			("alpha",        "float",    90.0, _("angle step (degrees)")),
			("nbrepeat",        "int",    2, _("nb repeat including original")),
			("keep",        "bool",    True, _("Keep original yes/no")),
		]
		self.buttons.append("exe")

	# ----------------------------------------------------------------------
	def execute(self, app):
		n = self["name"]
		xcenter = self["xcenter"]
		ycenter = self["ycenter"]
		alpha = self["alpha"]
		nbrepeat = self["nbrepeat"]
		if nbrepeat ==1 :
			nbrepeat =2
		keep = self["keep"]
		blocks = app.editor.getSelectedBlocks()
		if not blocks:
			app.editor.selectAll()
			blocks = app.editor.getSelectedBlocks()
		if not blocks:
			tkMessageBox.showerror(_("Tile error"),
				_("No g-code blocks selected"))
			return
		pos = blocks[-1]	# insert position
		alpha_current = alpha
		pos += 1
		for index in range(int(nbrepeat-1)):
			# clone selected blocks
			undoinfo = []
			newblocks = []
			for bid in blocks:
				undoinfo.append(app.gcode.cloneBlockUndo(bid, pos))
				newblocks.append((pos,None))
				pos += 1
			app.addUndo(undoinfo)
			app.gcode.rotateLines(newblocks,alpha_current, xcenter, ycenter)
			alpha_current+=alpha
		if not keep :
			app.editor.deleteBlock()
		app.refresh()
		app.setStatus(_("Rotated selected blocks"))

if __name__=="__main__":
	pass



#!/usr/bin/python
# -*- coding: ascii -*-

# Author: @harvie Tomas Mudrunka
# Date: 25 sept 2018

from __future__ import absolute_import
from __future__ import print_function
__author__ = "@harvie Tomas Mudrunka"
#__email__  = ""

__name__ = _("DragKnife")
__version__ = "0.3.0"

import math
import os.path
import re
from CNC import CNC,Block
from bmath import Vector
from bpath import eq, Path, Segment
from ToolsPage import Plugin
from math import pi, sqrt, sin, cos, asin, acos, atan2, hypot, degrees, radians, copysign, fmod


class Tool(Plugin):
	__doc__ = _("""Drag knife postprocessor""")			#<<< This comment will be show as tooltip for the ribbon button

	def __init__(self, master):
		Plugin.__init__(self, master,"DragKnife")
		self.icon = "dragknife"			#<<< This is the name of file used as icon for the ribbon button. It will be search in the "icons" subfolder
		self.group = "CAM"	#<<< This is the name of group that plugin belongs
		#self.oneshot = True
		#Here we are creating the widgets presented to the user inside the plugin
		#Name, Type , Default value, Description
		self.variables = [			#<<< Define a list of components for the GUI
			("name"    ,    "db" ,    "", _("Name")),							#used to store plugin settings in the internal database
			("offset", "mm", 3, _("dragknife offset"), _("distance from dragknife rotation center to the tip of the blade")),
			("angle", "float", 20, _("angle threshold"), _("do not perform pivot action for angles smaller than this")),
			("swivelz", "mm", 0, _("swivel height"), _("retract to this height for pivots (useful for thick materials, you should enter number slightly lower than material thickness)")),
			("initdir", "X+,Y+,Y-,X-,none", "X+", _("initial direction"), _("direction that knife blade is facing before and after cut. Eg.: if you set this to X+, then the knifes rotation axis should be on the right side of the tip. Meaning that the knife is ready to cut towards right immediately without pivoting. If you cut multiple shapes in single operation, it's important to have this set consistently across all of them.")),
			("feed", "mm", 200, _("feedrate")),
			("simulate", "bool", False, _("simulate"), _("Use this option to simulate cuting of dragknife path. Resulting shape will reflect what shape will actuall be cut. This should reverse the dragknife procedure and give you back the original shape from g-code that was previously processed for dragknife.")),
			("simpreci", "mm", 0.5, _("simulation precision"), _("Simulation is currently approximated by using lots of short lines. This is the length of these lines."))
		]
		self.buttons.append("exe")  #<<< This is the button added at bottom to call the execute method below

		self.help = """DragKnifes are special kind of razor/blade holders that can be fit into spindle of your CNC (do not turn the spindle on!!!). They are often used to cut soft and thin materials like vinyl stickers, fabric, leather, rubber gaskets, paper, cardboard, etc...

Dragknife blade is located off center to allow for automatic rotation (kinda like rear wheels of car pivot to the direction of front wheels).
This fact introduces the need for preprocessing the g-code to account with that offset. Otherwise it wouldn't be able to cut sharp corners. This plugin does this g-code postprocessing.
"""


	# ----------------------------------------------------------------------
	# This method is executed when user presses the plugin execute button
	# ----------------------------------------------------------------------
	def execute(self, app):
		dragoff = self.fromMm("offset")
		angleth = self["angle"]
		swivelz = self.fromMm("swivelz")
		initdir = self["initdir"]
		CNC.vars["cutfeed"] = self.fromMm("feed")
		simulate = self["simulate"]
		simpreci = self["simpreci"]

		def initPoint(P, dir, offset):
			P = Vector(P[0], P[1])

			if dir == 'X+':
				P[0]+=offset
			elif dir == 'X-':
				P[0]-=offset
			elif dir == 'Y+':
				P[1]+=offset
			elif dir == 'Y-':
				P[1]-=offset
			return P

		blocks  = []
		for bid in app.editor.getSelectedBlocks():
			if len(app.gcode.toPath(bid)) < 1: continue

			opath = app.gcode.toPath(bid)[0]
			npath = Path("dragknife %s: %s"%(dragoff,app.gcode[bid].name()))

			if not simulate:

				#Entry vector
				ventry = Segment(Segment.LINE, initPoint(opath[0].A, initdir, -dragoff), opath[0].A)

				#Exit vector
				vexit = Segment(Segment.LINE, opath[-1].B, initPoint(opath[-1].B, initdir, dragoff))
				opath.append(vexit)

				prevseg = ventry
				#Generate path with tangential lag for dragknife operation
				for i,seg in enumerate(opath):
					#Get adjacent tangential vectors in this point
					TA = prevseg.tangentEnd()
					TB = seg.tangentStart()

					#Compute difference between tangential vectors of two neighbor segments
					angle = degrees(acos(TA.dot(TB)))

					#Compute swivel direction
					arcdir = ( TA[0] * TB[1] ) - ( TA[1] * TB[0] )
					if arcdir < 0:
						arcdir = Segment.CW
					else:
						arcdir = Segment.CCW

					#Append swivel if needed (also always do entry/exit)
					if abs(angle) > angleth or (abs(angle) > 1 and ( i == 0 or i == len(opath)-1 )):
						arca = Segment(arcdir, prevseg.tangentialOffset(dragoff).B, seg.tangentialOffset(dragoff).A, prevseg.B)

						if swivelz !=0: arca._inside = [swivelz]
						npath.append(arca)

					#Append segment with tangential offset
					if i < len(opath)-1:
						npath.append(seg.tangentialOffset(dragoff))

					prevseg = seg

			elif simulate:

				opath = opath.linearize(simpreci, True)
				prevknife = initPoint(opath[0].A, initdir, -dragoff)
				for seg in opath:
					dist = sqrt((seg.B[0]-prevknife[0])**2+(seg.B[1]-prevknife[1])**2)
					move = ( seg.B - prevknife ).unit() * ( dist - dragoff )
					newknife = prevknife + move
					if not eq(newknife, prevknife):
						npath.append(Segment(Segment.LINE, prevknife, newknife))
					prevknife = newknife


			eblock = app.gcode.fromPath(npath)
			blocks.append(eblock)



		#active = app.activeBlock()
		#if active == 0: active+=1
		active=-1 #add to end
		app.gcode.insBlocks(active, blocks, "Dragknife") #<<< insert blocks over active block in the editor
		app.refresh()                                                                                           #<<< refresh editor
		app.setStatus(_("Generated: Dragknife"))                           #<<< feed back result
		#app.gcode.blocks.append(block)



#!/usr/bin/python
# -*- coding: ascii -*-
# $Id$
#
# Author:	DodoLaSaumure
# Date:	30-Dec-2019

from __future__ import absolute_import
from __future__ import print_function
__author__ = "DodoLaSaumure"
__email__  = ""


import math
from bmath import Vector
from CNC import CW,CNC,Block
from ToolsPage import Plugin


#==============================================================================
# SimpleArc class
#==============================================================================
class SimpleArc:
	def __init__(self, name): 
		self.name = name

	def calc(self, xcenter,ycenter,radius,startangle,endangle):
		self.Points=[]
		xcenter,ycenter,radius,startangle,endangle=float(xcenter),float(ycenter),abs(float(radius)),float(startangle),float(endangle)
		xstart=xcenter+radius*math.cos(startangle*math.pi/180.0)
		xend = xcenter+radius*math.cos(endangle*math.pi/180.0)
		ystart=ycenter+radius*math.sin(startangle*math.pi/180.0)
		yend=ycenter+radius*math.sin(endangle*math.pi/180.0)
		i=xcenter-xstart
		j=ycenter-ystart
		blocks = []
		block = Block(self.name)
		block.append(CNC.grapid(x=xstart,y=ystart))
		block.append(CNC.grapid(z=0.0))
		block.append("(entered)")
		if startangle<endangle :
			direction = 3
		else :
			direction = 2
		block.append(CNC.garc(direction,x=xend,y=yend,i=i,j=j))
		block.append("(exiting)")
		block.append(CNC.grapid(z=CNC.vars["safe"]))
		blocks.append(block)
		return blocks


#==============================================================================
# Create a simple Arc
#==============================================================================
class Tool(Plugin):
	__doc__ = _("Generate a simple Arc")

	def __init__(self, master):
		Plugin.__init__(self, master, "SimpleArc")
		self.icon  = "SimpleArc"
		self.group = "Generator"
		self.variables = [
			("xcenter",        "float",    0.0, _("Center X")),
			("ycenter",        "float",    0.0, _("Center Y")),
			("radius",        "float",    0, _("Radius")),
			("startangle",        "float",    0, _("Start Angle in Degrees")),
			("endangle",        "float",    360, _("End Angle in Degrees ")),
		]
		self.buttons.append("exe")
	# ----------------------------------------------------------------------
	def execute(self, app):
		n = self["name"]
		if not n or n=="default":
			n="SimpleArc"
		simpleArc = SimpleArc(n)
		blocks = simpleArc.calc(self["xcenter"], self["ycenter"], self["radius"], self["startangle"], self["endangle"])
		active = app.activeBlock()
		if active==0:
			active=1
		app.gcode.insBlocks(active, blocks, _("Create Simple Arc"))
		app.refresh()
		app.setStatus(_("Generated: Simple Arc"))

if __name__=="__main__":
	simpleArc = SimpleArc()
	simpleArc.calc(0,10,20,0,360)
	simpleArc.calc(10,10,20,0,-360)



#!/usr/bin/python
# -*- coding: ascii -*-

# Author: @harvie Tomas Mudrunka
# Date: 11 july 2018

from __future__ import absolute_import
from __future__ import print_function
__author__ = "@harvie Tomas Mudrunka"
#__email__  = ""

__name__ = _("slicemesh")
__version__ = "0.0.6"

#import math
import os.path
#import re
from CNC import CNC,Block
from ToolsPage import Plugin
#from math import pi, sqrt, sin, cos, asin, acos, atan2, hypot, degrees, radians, copysign, fmod

#FIXME: not sure how to force bCNC to prefer importing from bCNC/lib to importing from system
#	if having conflicts with system libs, you can try this. It helped for me:
#	pip2 uninstall meshcut stl ply itertools utils python-utils
#	pip2 install scipy numpy

#If needed trimesh supports following formats:
#binary/ASCII STL, Wavefront OBJ, ASCII OFF, binary/ASCII PLY, GLTF/GLB 2.0, 3MF, XAML, 3DXML, etc.
#https://github.com/mikedh/trimesh
#but it depends on numpy, scipy and networkx

import os
import numpy as np
#import numpy.linalg as la
#import itertools
#import utils
import meshcut
import ply #FIXME: write PLY parser which supports binary PLY files (currently can only do ASCII PLY)
import stl #FIXME: write smaller STL parser


class Tool(Plugin):
	__doc__ = _("""STL/PLY Slicer""")			#<<< This comment will be show as tooltip for the ribbon button

	def __init__(self, master):
		Plugin.__init__(self, master,"Slice Mesh")
		#Helical_Descent: is the name of the plugin show in the tool ribbon button
		self.icon = "mesh"			#<<< This is the name of file used as icon for the ribbon button. It will be search in the "icons" subfolder
		self.group = "CAM"	#<<< This is the name of group that plugin belongs
		#Here we are creating the widgets presented to the user inside the plugin
		#Name, Type , Default value, Description
		self.variables = [			#<<< Define a list of components for the GUI
			("name"    ,    "db" ,    "", _("Name")),							#used to store plugin settings in the internal database
			("file"    ,    "file" ,    "", _(".STL/.PLY file to slice"), "What file to slice"),
			("flat"    ,    "bool" ,    True, _("Get flat slice"), "Pack all slices into single Z height?"),
			("cam3d"    ,    "bool" ,    False, _("3D slice (devel)"), "This is just for testing"),
			("faceup"    ,    "Z,-Z,X,-X,Y,-Y" ,    "Z", _("Flip upwards"), "Which face goes up?"),
			("scale"    ,    "float" ,    1, _("scale factor"), "Size will be multiplied by this factor"),
			("zoff"  ,    "mm" ,    0, _("z offset"), "This will be added to Z"),
			("zstep"    ,    "mm" ,    0.1, _("layer height (0 = only single zmin)"), "Distance between layers of slices"),
			("zmin"    ,    "mm" ,    -1, _("minimum Z height"), "Height to start slicing"),
			("zmax"    ,    "mm" ,    1, _("maximum Z height"), "Height to stop slicing")
		]
		self.buttons.append("exe")  #<<< This is the button added at bottom to call the execute method below
		self.help = '''This plugin can slice meshes
#mesh

Currently it supports following formats:
STL (Binary and ASCII)
PLY (ASCII only)

You can use external software to edit and convert meshes to desired format.
It can also help you to visualy inspect your meshes before slicing.
Example of useful free software for working with meshes is called "MeshLab".

WARNING:

Recently we've noticed bug #1223, which causes this plugin to fail with some big(-ish) STL files.
Luckily PLY import is unaffected, so if you experience troubles, you can try to convert your file to a different format.

Also bCNC lacks proper 3D CAM features, so you can only apply 2D CAM operations on resulting slices.
Also note you can export resulting slices to DXF and SVG if needed.
'''


	# ----------------------------------------------------------------------
	# This method is executed when user presses the plugin execute button
	# ----------------------------------------------------------------------
	def execute(self, app):
		self.app = app
		file = self["file"]
		zstep = float(self["zstep"])
		zmin = float(self["zmin"])
		zmax = float(self["zmax"])
		flat = self["flat"]
		faceup = self["faceup"]
		scale = float(self["scale"])
		zoff = float(self["zoff"])
		cam3d = self["cam3d"]

		zout = None
		if flat: zout = 0

		blocks = []

		#Load mesh
		self.app.setStatus(_("Loading mesh: %s"%(file)), True)
		verts, faces = self.loadMesh(file)

		#Rotate/flip mesh
		if faceup == 'Z':
			pass
		elif faceup == '-Z':
			self.transformMesh(verts, 2, 2, -1, -1)
		elif faceup == 'X':
			self.transformMesh(verts, 2, 0,  1,  1)
		elif faceup == '-X':
			self.transformMesh(verts, 2, 0,  1, -1)
		elif faceup == 'Y':
			self.transformMesh(verts, 2, 1,  1,  1)
		elif faceup == '-Y':
			self.transformMesh(verts, 2, 1,  1, -1)

		if scale != 1 or zoff != 0:
			#FIXME: maybe use some numpy magic like verts = verts*scale ?
			for vert in verts:
				vert[0], vert[1], vert[2] = vert[0]*scale, vert[1]*scale, (vert[2]*scale)+zoff

		axes = ['z']
		if cam3d: axes = ['x','y','z']

		#Slice
		for axis in axes:
			if zstep <= 0:
				#cut only single layer if zstep <= 0
				zmax = zmin
				zstep = 1
			zmin, zmax = min(zmin,zmax), max(zmin,zmax) #make sure zmin<zmax
			#loop over multiple layers if zstep > 0
			z = zmax
			while z >= zmin:
				#print(_("Slicing %f / %f"%(z,zmax)))
				self.app.setStatus(_("Slicing %s %f in %f -> %f of %s"%(axis,z,zmin,zmax,file)), True)
				block = self.slice(verts, faces, z, zout, axis)
				if block is not None: blocks.append(block)
				z -= abs(zstep)

		#Insert blocks to bCNC
		active = app.activeBlock()
		app.gcode.insBlocks(active, blocks, "Mesh sliced") #<<< insert blocks over active block in the editor
		app.refresh()                                                                                           #<<< refresh editor
		app.setStatus(_("Mesh sliced"))                           #<<< feed back result


	def loadMesh(self, file):
		#Decide on filetype
		fn, fext = os.path.splitext(file)
		fext = fext.upper()

		if fext=='.STL':
			verts, faces = self.load_stl(file)
		elif fext=='.PLY':
			with open(file) as f:
				verts, faces, _ = ply.load_ply(f)
		else:
			print("unknown file extension",fext)
			return None

		return verts, faces


	#Rotate or flip mesh
	def transformMesh(self, verts, a, b=0, ia=1, ib=1):
		#FIXME: use numpy vectorization?
		for vert in verts:
			vert[a], vert[b] = ia*vert[b], ib*vert[a]


	def slice(self, verts, faces, z, zout=None, axis='z'):
		tags = '[slice]'
		if axis=='z': tags = '[slice,minz:%f]'%(float(z))
		block = Block("slice %s%f %s"%(axis,float(z),tags))

		#FIXME: slice along different axes
		if axis == 'x':
			plane_orig = (z, 0, 0)
			plane_norm = (1, 0, 0)
		elif axis == 'y':
			plane_orig = (0, z, 0)
			plane_norm = (0, 1, 0)
		else:
			plane_orig = (0, 0, z) #z height to slice
			plane_norm = (0, 0, 1)

		#Crosscut
		contours = meshcut.cross_section(verts, faces, plane_orig, plane_norm)

		#Flatten contours
		if zout is not None:
			for contour in contours:
				for segment in contour:
					segment[2] = zout

		#Contours to G-code
		for contour in contours:
			#print(contour)
			gtype = 0
			for segment in contour:
				block.append("g%s x%f y%f z%f"%(gtype, segment[0],segment[1],segment[2]))
				gtype = 1
			block.append("g1 x%f y%f z%f"%(contour[0][0],contour[0][1],contour[0][2])) #Close shape
			block.append("( ---------- cut-here ---------- )")
		if block: del block[-1]

		if not block: block = None
		return block

	def vert_dist(self, A, B):
		#return np.sqrt(np.sum(np.square(B-A)))
		return ((B[0]-A[0])**2+(B[1]-A[1])**2+(B[2]-A[2])**2)**(1.0/2)

	def vert_dist_matrix(self, verts):
		#FIXME: This is VERY SLOW:
		D = np.empty((len(verts), len(verts)), dtype=np.float64)
		for i,v in enumerate(verts):
			self.app.setStatus(_("Calculating distance %d of %d (SciPy not installed => using SLOW fallback method)"%(i,len(verts))), True)
			D[i] = D[:,i] = np.sqrt(np.sum(np.square(verts-verts[i]), axis=1))
			#for j in range(i,len(verts)):
			#	D[j][i] = D[i][j] = self.vert_dist(v,verts[j])
			#	#D[j][i] = D[i][j] = la.norm(verts[j]-v)
		return D

	def pdist_squareformed_numpy(self, a):
		#Thanks to Divakar Roy (@droyed) https://stackoverflow.com/questions/52030458/vectorized-spatial-distance-in-python-using-numpy
		a_sumrows = np.einsum('ij,ij->i',a,a)
		dist = a_sumrows[:,None] + a_sumrows -2*np.dot(a,a.T)
		np.fill_diagonal(dist,0)
		return dist

	def merge_close_vertices(self, verts, faces, close_epsilon=1e-5):
		"""
		Will merge vertices that are closer than close_epsilon.

		Warning, this has a O(n^2) memory usage because we compute the full
		vert-to-vert distance matrix. If you have a large mesh, might want
		to use some kind of spatial search structure like an octree or some fancy
		hashing scheme

		Returns: new_verts, new_faces
		"""
		# Pairwise distance between verts
		verts = np.array(verts, dtype=np.float64)
		#Use SciPy, otherwise use slow fallback
		try:
			import scipy.spatial.distance as spdist
			D = spdist.cdist(verts, verts)
		except ImportError:
			D = np.sqrt(np.abs(self.pdist_squareformed_numpy(verts)))

		#Test
		print(len(verts), len(D), len(D[0]))
		#print(D)
		#print(spdist.cdist(verts, verts))

		# Compute a mapping from old to new : for each input vert, store the index
		# of the new vert it will be merged into
		close_epsilon = 1e-5
		old2new = np.zeros(D.shape[0], dtype=np.int)
		# A mask indicating if a vertex has already been merged into another
		merged_verts = np.zeros(D.shape[0], dtype=np.bool)
		new_verts = []
		for i in range(D.shape[0]):
			if merged_verts[i]:
				continue
			else:
				# The vertices that will be merged into this one
				merged = np.flatnonzero(D[i, :] < close_epsilon)
				old2new[merged] = len(new_verts)
				new_verts.append(verts[i])
				merged_verts[merged] = True

		new_verts = np.array(new_verts)

		# Recompute face indices to index in new_verts
		new_faces = np.zeros((len(faces), 3), dtype=np.int)
		for i, f in enumerate(faces):
			new_faces[i] = (old2new[f[0]], old2new[f[1]], old2new[f[2]])

		# again, plot with utils.trimesh3d(new_verts, new_faces)
		return new_verts, new_faces


	def load_stl(self, stl_fname):
		#import stl
		m = stl.mesh.Mesh.from_file(stl_fname)

		# Flatten our vert array to Nx3 and generate corresponding faces array
		verts = m.vectors.reshape(-1, 3)
		faces = np.arange(len(verts)).reshape(-1, 3)

		verts, faces = self.merge_close_vertices(verts, faces)
		return verts, faces



# -*- coding: ascii -*-
# $Id$
#
# Author: Vasilis Vlachoudis
#  Email: Vasilis.Vlachoudis@cern.ch
#   Date: 16-Apr-2015
from __future__ import absolute_import
from __future__ import print_function
__author__ = "Vasilis Vlachoudis"
__email__  = "vvlachoudis@gmail.com"
__version__ = "0.9.14-dev"
__date__    = "8 Jan 2019"
__prg__     = "bCNC"

import sys
import os
import sys
import glob
import traceback
import gettext

__platform_fingerprint__ = "(%s py%s.%s.%s)"%(
		sys.platform, sys.version_info.major,
		sys.version_info.minor, sys.version_info.micro)
__title__   = "%s %s %s"%(__prg__, __version__, __platform_fingerprint__)

__all__ = (
	# Override builtins
	"basestring","unicode","unichr","bytes","str",

	# Utility functions in here
	"to_unicode","to_zip"

	"OLD", # Leave this ol'comma alone!
	)

#------------------------------------------------------------------------------
# We only need one try here for compatability check (for the moment at least!)
OLD = False
try:
	# old version
	unicode
except (NameError, AttributeError):
	# newer version
	str        = str
	unichr     = chr          # pylint: disable=redefined-builtin,invalid-name,unichr-builtin
	unicode    = str          # pylint: disable=redefined-builtin,invalid-name,unicode-builtin
	bytes      = bytes
	basestring = (str, bytes) # pylint: disable=redefined-builtin,invalid-name,basestring-builtin
else:
	# old version
	str        = str
	unichr     = unichr
	unicode    = unicode
	bytes      = str
	basestring = basestring
	OLD = True

#------------------------------------------------------------------------------
# Standard imports
if OLD:
	from Tkinter import *
	import tkFont
	import tkMessageBox
	import ConfigParser
	import __builtin__
	def to_unicode(s):
		if isinstance(s, (unicode)) or type(s) == 'unicode':
			s = s.encode('utf-8')
		return s
	def to_zip(*args,**kwargs):
		return zip(*args,**kwargs)
	def to_decode(s):
		if isinstance(s, bytes):
			return s.decode("utf-8")
		else :
			return s

else:
	from tkinter import *
	import tkinter.font as tkFont
	import tkinter.messagebox as tkMessageBox
	import configparser as ConfigParser
	import builtins as __builtin__
	def to_unicode(s): return s
	def to_zip(*args,**kwargs): return (list(zip(*args,**kwargs)))
	def to_decode(s):
		return s
# try:
# 	import __builtin__
# except:
# 	import builtins as __builtin__
# 	#__builtin__.unicode = str		# Dirty hack for Python 3

try:
	import serial
except:
	serial = None

__prg__     = "bCNC"
prgpath   = os.path.abspath(os.path.dirname(__file__))
if getattr( sys, 'frozen', False ):
	#When being bundled by pyinstaller, paths are different
	print("Running as pyinstaller bundle!", sys.argv[0])
	prgpath   = os.path.abspath(os.path.dirname(sys.argv[0]))
iniSystem = os.path.join(prgpath,"%s.ini"%(__prg__))
iniUser   = os.path.expanduser("~/.%s" % (__prg__))
hisFile   = os.path.expanduser("~/.%s.history" % (__prg__))

# dirty way of substituting the "_" on the builtin namespace
#__builtin__.__dict__["_"] = gettext.translation('bCNC', 'locale', fallback=True).ugettext
__builtin__._ = gettext.translation('bCNC', os.path.join(prgpath,'locale'), fallback=True).gettext
__builtin__.N_ = lambda message: message

from log import say
import Ribbon
import tkExtra

__www__     = "https://github.com/vlachoudis/bCNC"
__contribute__ = \
		"@effer Filippo Rivato\n" \
		"@carlosgs Carlos Garcia Saura\n" \
		"@dguerizec\n" \
		"@buschhardt\n" \
		"@MARIOBASZ\n" \
		"@harvie Tomas Mudrunka"
__credits__ = \
		"@1bigpig\n" \
		"@chamnit Sonny Jeon\n" \
		"@harvie Tomas Mudrunka\n" \
		"@onekk Carlo\n" \
		"@SteveMoto\n" \
		"@willadams William Adams"
__translations__ = \
		"Dutch - @hypothermic\n" \
		"French - @ThierryM\n" \
		"German - @feistus, @SteveMoto\n" \
		"Italian - @onekk\n" \
		"Japanese - @stm32f1\n" \
		"Korean - @jjayd\n" \
		"Portuguese - @moacirbmn \n" \
		"Russian - @minithc\n" \
		"Simplified Chinese - @Bluermen\n" \
		"Spanish - @carlosgs\n" \
		"Traditional Chinese - @Engineer2Designer"

LANGUAGES = {
		""      : "<system>",
		"de"    : "Deutsch",
		"en"    : "English",
		"es"    : u"Espa\u00f1ol",
		"fr"    : u"Fran\u00e7ais",
		"it"    : "Italiano",
		"ja"    : "Japanese",
		"kr"    : "Korean",
		"nl"    : "Nederlands",
		"pt_BR" : "Brazilian - Portuguese",
		"ru"    : "Russian",
		"zh_cn" : "Simplified Chinese",
		"zh_tw" : "Traditional Chinese",
	}

icons     = {}
images     = {}
config    = ConfigParser.ConfigParser()
print("new-config", __name__, config) #This is here to debug the fact that config is sometimes instantiated twice
language  = ""

_errorReport = True
errors       = []
_maxRecent   = 10

_FONT_SECTION = "Font"


#New class to provide config for everyone
#FIXME: create single instance of this and pass it to all parts of application
class Config():
	def greet(self, who=__name__):
		print("Config class loaded in %s"%(who))


#------------------------------------------------------------------------------
def loadIcons():
	global icons
	icons = {}
	for img in glob.glob("%s%sicons%s*.gif"%(prgpath,os.sep,os.sep)):
		name,ext = os.path.splitext(os.path.basename(img))
		try:
			icons[name] = PhotoImage(file=img)
			if getBool("CNC", "doublesizeicon"):
				icons[name] = icons[name].zoom(2,2)
		except TclError:
			pass

	#Images
	global images
	images = {}
	for img in glob.glob("%s%simages%s*.gif"%(prgpath,os.sep,os.sep)):
		name,ext = os.path.splitext(os.path.basename(img))
		try:
			images[name] = PhotoImage(file=img)
			if getBool("CNC", "doublesizeicon"):
				images[name] = images[name].zoom(2,2)
		except TclError:
			pass


#------------------------------------------------------------------------------
def delIcons():
	global icons
	if len(icons) > 0:
		for i in icons.values():
			del i
		icons = {}	# needed otherwise it complains on deleting the icons

	global images
	if len(images) > 0:
		for i in images.values():
			del i
		images = {}	# needed otherwise it complains on deleting the icons


#------------------------------------------------------------------------------
# Load configuration
#------------------------------------------------------------------------------
def loadConfiguration(systemOnly=False):
	global config, _errorReport, language
	if systemOnly:
		config.read(iniSystem)
	else:
		config.read([iniSystem, iniUser])
		_errorReport = getInt("Connection","errorreport",1)

		language = getStr(__prg__, "language")
		if language:
			# replace language
			__builtin__._ = gettext.translation('bCNC', os.path.join(prgpath,'locale'),
					fallback=True, languages=[language]).gettext


#------------------------------------------------------------------------------
# Save configuration file
#------------------------------------------------------------------------------
def saveConfiguration():
	global config
	cleanConfiguration()
	f = open(iniUser,"w")
	config.write(f)
	f.close()
	delIcons()


#----------------------------------------------------------------------
# Remove items that are the same as in the default ini
#----------------------------------------------------------------------
def cleanConfiguration():
	global config
	newconfig = config	# Remember config
	config = ConfigParser.ConfigParser()

	loadConfiguration(True)

	# Compare items
	for section in config.sections():
		for item, value in config.items(section):
			try:
				new = newconfig.get(section, item)
				if value==new:
					newconfig.remove_option(section, item)
			except ConfigParser.NoOptionError:
				pass
	config = newconfig


#------------------------------------------------------------------------------
# add section if it doesn't exist
#------------------------------------------------------------------------------
def addSection(section):
	global config
	if not config.has_section(section):
		config.add_section(section)


#------------------------------------------------------------------------------
def getStr(section, name, default=""):
	global config
	try:
		return config.get(section, name)
	except:
		return default


#------------------------------------------------------------------------------
def getUtf(section, name, default=""):
	global config
	try:
		return config.get(section, name)
	except:
		return default


#------------------------------------------------------------------------------
def getInt(section, name, default=0):
	global config
	try: return int(config.get(section, name))
	except: return default


#------------------------------------------------------------------------------
def getFloat(section, name, default=0.0):
	global config
	try: return float(config.get(section, name))
	except: return default


#------------------------------------------------------------------------------
def getBool(section, name, default=False):
	global config
	try: return bool(int(config.get(section, name)))
	except: return default


#-------------------------------------------------------------------------------
# Return a font from a string
#-------------------------------------------------------------------------------
def makeFont(name, value=None):
	try:
		font = tkFont.Font(name=name, exists=True)
	except TclError:
		font = tkFont.Font(name=name)
		font.delete_font = False
	except AttributeError:
		return None

	if value is None: return font

	if isinstance(value, tuple):
		font.configure(family=value[0])
		try:    font.configure(size=value[1])
		except: pass
		try:    font.configure(weight=value[2])
		except: pass
		try:    font.configure(slant=value[3])
		except: pass
	return font


#-------------------------------------------------------------------------------
# Create a font string
#-------------------------------------------------------------------------------
def fontString(font):
	name  = str(font[0])
	size  = str(font[1])
	if name.find(' ')>=0:
		s = '"%s" %s'%(name,size)
	else:
		s = '%s %s'%(name,size)

	try:
		if font[2] == tkFont.BOLD: s += " bold"
	except: pass
	try:
		if font[3] == tkFont.ITALIC: s += " italic"
	except: pass
	return s


#-------------------------------------------------------------------------------
# Get font from configuration
#-------------------------------------------------------------------------------
def getFont(name, default=None):
	try:
		value = config.get(_FONT_SECTION, name)
	except:
		value = None

	if not value:
		font = makeFont(name, default)
		setFont(name, font)
		return font

	if isinstance(value, str):
		value = tuple(value.split(','))

	if isinstance(value, tuple):
		font = makeFont(name, value)
		if font is not None: return font
	return value


#-------------------------------------------------------------------------------
# Set font in configuration
#-------------------------------------------------------------------------------
def setFont(name, font):
	if font is None: return
	if isinstance(font,str):
		config.set(_FONT_SECTION, name, font)
	elif isinstance(font,tuple):
		config.set(_FONT_SECTION, name, ",".join(map(str,font)))
	else:
		config.set(_FONT_SECTION, name, "%s,%s,%s" % \
			(font.cget("family"),font.cget("size"),font.cget("weight")))


#------------------------------------------------------------------------------
def setBool(section, name, value):
	global config
	config.set(section, name, str(int(value)))


#------------------------------------------------------------------------------
def setStr(section, name, value):
	global config
	config.set(section, name, str(value))


#------------------------------------------------------------------------------
def setUtf(section, name, value):
	global config
	try:
		s = str(value)
	except:
		s = value
	config.set(section, name, s)

setInt   = setStr
setFloat = setStr


#-------------------------------------------------------------------------------
# Add Recent
#-------------------------------------------------------------------------------
def addRecent(filename):
	try:
		sfn = str(os.path.abspath(filename))
	except UnicodeEncodeError:
		sfn = filename

	last = _maxRecent-1
	for i in range(_maxRecent):
		rfn = getRecent(i)
		if rfn is None:
			last = i-1
			break
		if rfn == sfn:
			if i==0: return
			last = i-1
			break

	# Shift everything by one
	for i in range(last, -1, -1):
		config.set("File", "recent.%d"%(i+1), getRecent(i))
	config.set("File", "recent.0", sfn)


#-------------------------------------------------------------------------------
def getRecent(recent):
	try:
		return config.get("File","recent.%d"%(recent))
	except ConfigParser.NoOptionError:
		return None


#------------------------------------------------------------------------------
# Return all comports when serial.tools.list_ports is not available!
#------------------------------------------------------------------------------
def comports(include_links=True):
	locations=[	'/dev/ttyACM',
			'/dev/ttyUSB',
			'/dev/ttyS',
			'com']

	comports = []
	for prefix in locations:
		for i in range(32):
			device = "%s%d"%(prefix,i)
			try:
				os.stat(device)
				comports.append((device,None,None))
			except OSError:
				pass

			# Detects windows XP serial ports
			try:
				s = serial.Serial(device)
				s.close()
				comports.append((device,None,None))
			except:
				pass
	return comports


#===============================================================================
def addException():
	global errors
	#self.widget._report_exception()
	try:
		typ, val, tb = sys.exc_info()
		traceback.print_exception(typ, val, tb)
		if errors: errors.append("")
		exception = traceback.format_exception(typ, val, tb)
		errors.extend(exception)
		if len(errors) > 100:
			# If too many errors are found send the error report
			ReportDialog(self.widget)
	except:
		say(str(sys.exc_info()))


#===============================================================================
class CallWrapper:
	"""Replaces the Tkinter.CallWrapper with extra functionality"""
	def __init__(self, func, subst, widget):
		"""Store FUNC, SUBST and WIDGET as members."""
		self.func   = func
		self.subst  = subst
		self.widget = widget

	# ----------------------------------------------------------------------
	def __call__(self, *args):
		"""Apply first function SUBST to arguments, than FUNC."""
		try:
			if self.subst:
				args = self.subst(*args)
			return self.func(*args)
		# One possible fix is to make an external file for the wrapper
		# and import depending the version
		#except SystemExit, msg:	# python2.4 syntax
		#except SystemExit as msg:	# python3 syntax
		#	raise SystemExit(msg)
		except SystemExit:		# both
			raise SystemExit(sys.exc_info()[1])
		except KeyboardInterrupt:
			pass
		except:
			addException()


#===============================================================================
# Error message reporting dialog
#===============================================================================
class ReportDialog(Toplevel):
	_shown = False	# avoid re-entry when multiple errors are displayed

	def __init__(self, master):
		if ReportDialog._shown: return
		ReportDialog._shown = True

		Toplevel.__init__(self, master)
		if master is not None: self.transient(master)
		self.title(_("Error Reporting"))

		# Label Frame
		frame = LabelFrame(self, text=_("Report"))
		frame.pack(side=TOP, expand=YES, fill=BOTH)

		l = Label(frame, text=_("The following report is about to be send "\
				"to the author of %s")%(__name__), justify=LEFT, anchor=W)
		l.pack(side=TOP)

		self.text = Text(frame, background=tkExtra.GLOBAL_CONTROL_BACKGROUND)
		self.text.pack(side=LEFT, expand=YES, fill=BOTH)

		sb = Scrollbar(frame, orient=VERTICAL, command=self.text.yview)
		sb.pack(side=RIGHT, fill=Y)
		self.text.config(yscrollcommand=sb.set)

		# email frame
		frame = Frame(self)
		frame.pack(side=TOP, fill=X)

		l = Label(frame, text=_("Your email"))
		l.pack(side=LEFT)

		self.email = Entry(frame, background=tkExtra.GLOBAL_CONTROL_BACKGROUND)
		self.email.pack(side=LEFT, expand=YES, fill=X)

		# Automatic error reporting
		self.err = BooleanVar()
		self.err.set(_errorReport)
		b = Checkbutton(frame, text=_("Automatic error reporting"),
			variable=self.err, anchor=E, justify=RIGHT)
		b.pack(side=RIGHT)

		# Buttons
		frame = Frame(self)
		frame.pack(side=BOTTOM, fill=X)

		b = Button(frame, text=_("Close"),
				compound=LEFT,
				command=self.cancel)
		b.pack(side=RIGHT)
		b = Button(frame, text=_("Send report"),
				state=DISABLED, #Error reporting endpoint is currently offline (#824), disabled this to avoid timeout and confusion
				compound=LEFT,
				command=self.send)
		b.pack(side=RIGHT)

		from bCNC import __version__, __date__

		# Fill report
		txt = [ "Program     : %s"%(__prg__),
			"Version     : %s"%(__version__),
			"Last Change : %s"%(__date__),
			"Platform    : %s"%(sys.platform),
			"Python      : %s"%(sys.version),
			"TkVersion   : %s"%(TkVersion),
			"TclVersion  : %s"%(TclVersion),
			"\nTraceback:" ]
		for e in errors:
			if e!="" and e[-1] == "\n":
				txt.append(e[:-1])
			else:
				txt.append(e)

		self.text.insert('0.0', "\n".join(txt))

		# Guess email
		user = os.getenv("USER")
		host = os.getenv("HOSTNAME")
		if user and host:
			email = "%s@%s"%(user,host)
		else:
			email = ""
		self.email.insert(0,email)

		self.protocol("WM_DELETE_WINDOW", self.close)
		self.bind('<Escape>', self.close)

		# Wait action
		self.wait_visibility()
		self.grab_set()
		self.focus_set()
		self.wait_window()

	# ----------------------------------------------------------------------
	def close(self, event=None):
		ReportDialog._shown = False
		self.destroy()

	# ----------------------------------------------------------------------
	def send(self):
		import httplib, urllib
		global errors
		email = self.email.get()
		desc  = self.text.get('1.0', END).strip()

		# Send information
		self.config(cursor="watch")
		self.text.config(cursor="watch")
		self.update_idletasks()
		params = urllib.urlencode({"email":email, "desc":desc})
		headers = {"Content-type": "application/x-www-form-urlencoded",
			"Accept": "text/plain"}
		conn = httplib.HTTPConnection("www.bcnc.org:80")
		try:
			conn.request("POST", "/flair/send_email_bcnc.php", params, headers)
			response = conn.getresponse()
		except:
			tkMessageBox.showwarning(_("Error sending report"),
				_("There was a problem connecting to the web site"),
				parent=self)
		else:
			if response.status == 200:
				tkMessageBox.showinfo(_("Report successfully send"),
					_("Report was successfully uploaded to web site"),
					parent=self)
				del errors[:]
			else:
				tkMessageBox.showwarning(_("Error sending report"),
					_("There was an error sending the report\nCode=%d %s")%\
					(response.status, response.reason),
					parent=self)
		conn.close()
		self.config(cursor="")
		self.cancel()

	# ----------------------------------------------------------------------
	def cancel(self):
		global _errorReport, errors
		_errorReport = self.err.get()
		config.set("Connection", "errorreport", str(bool(self.err.get())))
		del errors[:]
		self.close()

	# ----------------------------------------------------------------------
	@staticmethod
	def sendErrorReport():
		ReportDialog(None)


#===============================================================================
# User Button
#===============================================================================
class UserButton(Ribbon.LabelButton):
	TOOLTIP  = "User configurable button.\n<RightClick> to configure"

	def __init__(self, master, cnc, button, *args, **kwargs):
		if button == 0:
			Button.__init__(self, master, *args, **kwargs)
		else:
			Ribbon.LabelButton.__init__(self, master, *args, **kwargs)
		self.cnc = cnc
		self.button = button
		self.get()
		#self.bind("<Control-Button-1>", self.edit)
		self.bind("<Button-3>",         self.edit)
		self.bind("<Control-Button-1>", self.edit)
		self["command"] = self.execute

	# ----------------------------------------------------------------------
	# get information from configuration
	# ----------------------------------------------------------------------
	def get(self):
		if self.button == 0: return
		name = self.name()
		self["text"] = name
		#if icon == "":
		#	icon = icons.get("empty","")
		self["image"] = icons.get(self.icon(),icons["material"])
		self["compound"] = LEFT
		tooltip = self.tooltip()
		if not tooltip: tooltip = UserButton.TOOLTIP
		tkExtra.Balloon.set(self, tooltip)

	# ----------------------------------------------------------------------
	def name(self):
		try:
			return config.get("Buttons","name.%d"%(self.button))
		except:
			return str(self.button)

	# ----------------------------------------------------------------------
	def icon(self):
		try:
			return config.get("Buttons","icon.%d"%(self.button))
		except:
			return None

	# ----------------------------------------------------------------------
	def tooltip(self):
		try:
			return config.get("Buttons","tooltip.%d"%(self.button))
		except:
			return ""

	# ----------------------------------------------------------------------
	def command(self):
		try:
			return config.get("Buttons","command.%d"%(self.button))
		except:
			return ""

	# ----------------------------------------------------------------------
	# Edit button
	# ----------------------------------------------------------------------
	def edit(self, event=None):
		UserButtonDialog(self, self)
		self.get()

	# ----------------------------------------------------------------------
	# Execute command
	# ----------------------------------------------------------------------
	def execute(self):
		cmd = self.command()
		if not cmd:
			self.edit()
			return
		for line in cmd.splitlines():
			self.cnc.pendant.put(line)


#===============================================================================
# User Configurable Buttons
#===============================================================================
class UserButtonDialog(Toplevel):
	NONE = "<none>"

	def __init__(self, master, button):
		Toplevel.__init__(self, master)
		self.title(_("User configurable button"))
		self.transient(master)
		self.button = button

		# Name
		row,col = 0,0
		Label(self, text=_("Name:")).grid(row=row, column=col, sticky=E)
		col += 1
		self.name = Entry(self, background=tkExtra.GLOBAL_CONTROL_BACKGROUND)
		self.name.grid(row=row, column=col, columnspan=2, sticky=EW)
		tkExtra.Balloon.set(self.name, _("Name to appear on button"))

		# Icon
		row,col = row+1,0
		Label(self, text=_("Icon:")).grid(row=row, column=col, sticky=E)
		col += 1
		self.icon = Label(self, relief=RAISED)
		self.icon.grid(row=row, column=col, sticky=EW)
		col += 1
		self.iconCombo = tkExtra.Combobox(self, True,
					width=5,
					command=self.iconChange)
		lst = list(sorted(icons.keys()))
		lst.insert(0,UserButtonDialog.NONE)
		self.iconCombo.fill(lst)
		self.iconCombo.grid(row=row, column=col, sticky=EW)
		tkExtra.Balloon.set(self.iconCombo, _("Icon to appear on button"))

		# Tooltip
		row,col = row+1,0
		Label(self, text=_("Tool Tip:")).grid(row=row, column=col, sticky=E)
		col += 1
		self.tooltip = Entry(self, background=tkExtra.GLOBAL_CONTROL_BACKGROUND)
		self.tooltip.grid(row=row, column=col, columnspan=2, sticky=EW)
		tkExtra.Balloon.set(self.tooltip, _("Tooltip for button"))

		# Tooltip
		row,col = row+1,0
		Label(self, text=_("Command:")).grid(row=row, column=col, sticky=N+E)
		col += 1
		self.command = Text(self, background=tkExtra.GLOBAL_CONTROL_BACKGROUND, width=40, height=10)
		self.command.grid(row=row, column=col, columnspan=2, sticky=EW)

		self.grid_columnconfigure(2,weight=1)
		self.grid_rowconfigure(row,weight=1)

		# Actions
		row += 1
		f = Frame(self)
		f.grid(row=row, column=0, columnspan=3, sticky=EW)
		Button(f, text=_("Cancel"), command=self.cancel).pack(side=RIGHT)
		Button(f, text=_("Ok"),     command=self.ok).pack(side=RIGHT)

		# Set variables
		self.name.insert(0,self.button.name())
		self.tooltip.insert(0,self.button.tooltip())
		icon = self.button.icon()
		if icon is None:
			self.iconCombo.set(UserButtonDialog.NONE)
		else:
			self.iconCombo.set(icon)
		self.icon["image"] = icons.get(icon,"")
		self.command.insert("1.0", self.button.command())

		# Wait action
		self.wait_visibility()
		self.grab_set()
		self.focus_set()
		self.wait_window()

	# ----------------------------------------------------------------------
	def ok(self, event=None):
		n = self.button.button
		config.set("Buttons", "name.%d"%(n), self.name.get().strip())
		icon = self.iconCombo.get()
		if icon == UserButtonDialog.NONE: icon = ""
		config.set("Buttons", "icon.%d"%(n), icon)
		config.set("Buttons", "tooltip.%d"%(n), self.tooltip.get().strip())
		config.set("Buttons", "command.%d"%(n), self.command.get("1.0",END).strip())
		self.destroy()

	# ----------------------------------------------------------------------
	def cancel(self):
		self.destroy()

	# ----------------------------------------------------------------------
	def iconChange(self):
		self.icon["image"] = icons.get(self.iconCombo.get(),"")



#!/usr/bin/python
# -*- coding: ascii -*-
# $Id$
#
# Author:	DodoLaSaumure
# Date:	30-Dec-2019

from __future__ import absolute_import
from __future__ import print_function
__author__ = "DodoLaSaumure"
__email__  = ""


import math
from bmath import Vector
from CNC import CW,CNC,Block
from ToolsPage import Plugin


#==============================================================================
# SimpleDrill class
#==============================================================================
class SimpleDrill:
	def __init__(self, name): 
		self.name = name
	def accelerateIfNeeded(self,ztogo,drillfeed):
		if self.safeZforG0>0:
			self.block.append(CNC.grapid(z=ztogo+self.safeZforG0))
			kwargs={"f":float(drillfeed)}
			self.block.append(CNC.gline(None,None,ztogo,**kwargs))

	def calc(self,x,y,depth,peck,dwell,drillFeed,safeZforG0):
		self.safeZforG0 =float(abs(safeZforG0))
		peck=abs(float(peck))
		currentz=0.0
		self.blocks = []
		self.block = Block(self.name)
		self.block.append(CNC.grapid(x=x,y=y))
		self.block.append(CNC.grapid(z=CNC.vars["safe"]))
		self.accelerateIfNeeded(0.0,drillFeed)
		self.block.append("(entered)")
		while(currentz>depth):
			currentz-=peck
			if currentz < depth:
				currentz = depth
			kwargs={"f":float(drillFeed)}
			self.block.append(CNC.gline(None,None,float(currentz),**kwargs))
			if self.safeZforG0 >0:
				self.block.append(CNC.grapid(z=0.0+self.safeZforG0))
			else :
				self.block.append(CNC.grapid(z=CNC.vars["safe"]))
			self.block.append("g4 %s"%(CNC.fmt("p",float(dwell))))
			if currentz > depth:
				self.accelerateIfNeeded(currentz,drillFeed)
		self.block.append("(exiting)")
		self.block.append(CNC.grapid(z=CNC.vars["safe"]))
		self.blocks.append(self.block)
		return self.blocks


#==============================================================================
# Create a simple drill
#==============================================================================
class Tool(Plugin):
	__doc__ = _("Generate a simple Drill")

	def __init__(self, master):
		Plugin.__init__(self, master, "SimpleDrill")
		self.icon  = "SimpleDrill"
		self.group = "Generator"
		self.variables = [
			("x",        "mm",    0, _("X")),
			("y",        "mm",    0, _("Y")),
			("depth",     "mm" ,    "", _("Target z (negative under surface)")),
			("peck",      "mm" ,    "", _("Peck depth (positive)")),
			("dwell",     "float" , 0, _("Dwell time (s)")),
			("drillFeed",      "mm/mn" ,    "", _("Z feed for drilling")),
			("safeZforG0",      "mm" ,    -1.0, _("Safe z secu for G0")),
			
			]
		self.help="""This plugin drills a hole, by going back and forth. when safe Z secu for G0 is -1 A drill sequence consists in making a G1 to the current depth, then G0 to default zSafe.
The depth is increased progressively by steps of "peck" value.
When a value is entered in safe Z secu, the tool will go G0 above the current depth, then will drill.
This can accelerate the process """
		self.buttons.append("exe")
	# ----------------------------------------------------------------------
	def execute(self, app):
		n = self["name"]
		if not n or n=="default":
			n="SimpleDrill"
		simpleDrill = SimpleDrill(n)
		blocks = simpleDrill.calc(self["x"], self["y"], self["depth"], self["peck"], self["dwell"],self["drillFeed"],self["safeZforG0"])
		active = app.activeBlock()
		if active==0:
			active=1
		app.gcode.insBlocks(active, blocks, _("Create Simple Drill"))
		app.refresh()
		app.setStatus(_("Generated: Simple Drill"))

if __name__=="__main__":
	simpleDrill = SimpleDrill()
	simpleDrill.calc(10,10,-20,1,0.2,50)



from setuptools import setup, find_packages
import sys #added for python2 support
print("Running bCNC setup...")

with open("README.md", "r") as fh:
	long_description = fh.read()

if sys.version_info[0] >= 3:
	opencv_version = '4.4.0.46' # Recent version for Puthon 3
else: #python version lower then 3 compatability
	opencv_version ='4.2.0.32' # use the last opencv version for python 2.7

setup(
	name = "bCNC",
	version = "0.9.14.313",
	license="GPLv2",
	description='Swiss army knife for all your CNC/g-code needs',
	long_description=long_description,
	long_description_content_type="text/markdown",
	packages = find_packages(),
	author = 'Vasilis Vlachoudis',
	author_email='vvlachoudis@gmail.com',
	url="https://github.com/vlachoudis/bCNC",
	include_package_data=True,
	#python_requires="<3.0",
	install_requires = [
		"pyobjc ; sys_platform == 'darwin'",
		"pyobjc-core; sys_platform == 'darwin'",
		"pyobjc-framework-Quartz; sys_platform == 'darwin'",

		"pyserial ; sys_platform != 'win32'",	#Windows XP can't handle pyserial newer than 3.0.1 (it can be installed, but does not work)
		"pyserial<=3.0.1 ; sys_platform == 'win32'",
		'numpy>=1.12',
		'Pillow>=4.0',
		'opencv-python==%s ; ("arm" not in platform_machine) and ("aarch64" not in platform_machine)' % (opencv_version),	#Note there are no PyPI OpenCV packages for ARM (Raspberry PI, Orange PI, etc...)
	],

	entry_points = {
		'console_scripts': [
			#'bCNC = {package}.{module}:{main_function}',
			#'bCNC = bCNC.bCNC:main',
			'bCNC = bCNC.__main__:main',
		]
	},

	classifiers=[
		"Development Status :: 4 - Beta",
		"License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
		"Operating System :: OS Independent",
		"Topic :: Multimedia :: Graphics :: 3D Modeling",
		"Topic :: Multimedia :: Graphics :: Capture",
		"Topic :: Multimedia :: Graphics :: Editors :: Vector-Based",
		"Topic :: Multimedia :: Graphics :: Graphics Conversion",
		"Topic :: Multimedia :: Graphics :: Viewers",
		"Topic :: Scientific/Engineering",
		"Topic :: Scientific/Engineering :: Electronic Design Automation (EDA)",
		"Topic :: Terminals :: Serial",
		"Natural Language :: Dutch",
		"Natural Language :: English",
		"Natural Language :: German",
		"Natural Language :: Spanish",
		"Natural Language :: Portuguese",
		"Natural Language :: Portuguese (Brazilian)",
		"Natural Language :: French",
		"Natural Language :: Italian",
		"Natural Language :: Japanese",
		"Natural Language :: Korean",
		"Natural Language :: Russian",
		"Natural Language :: Chinese (Simplified)",
		"Natural Language :: Chinese (Traditional)",
	]
)



#!/usr/bin/python
# -*- coding: ascii -*-
# $Id$
#
# Author:	DodoLaSaumure
# Date:	30-Dec-2019

from __future__ import absolute_import
from __future__ import print_function
__author__ = "DodoLaSaumure"
__email__  = ""


import math
from bmath import Vector
from CNC import CW,CNC,Block
from ToolsPage import Plugin


#==============================================================================
# SimpleRectangle class
#==============================================================================
class SimpleRectangle:
	def __init__(self, name): 
		self.name = name

	def calc(self, xstart,ystart,xend,yend,radius,cw):
		self.Points=[]
		self.corners = [min(float(xstart),float(xend)),
						min(float(ystart),float(yend)),
						max(float(xstart),float(xend)),
						max(float(ystart),float(yend)),
						]
		
		xmin,ymin,xmax,ymax = self.corners[0],self.corners[1],self.corners[2],self.corners[3]
		r = min(radius,(xmax-xmin)/2,(ymax-ymin)/2)
		blocks = []
		block = Block(self.name)
		block.append(CNC.grapid(x=xmin,y=ymin+r))
		block.append(CNC.grapid(z=0.0))
		block.append("(entered)")
		if cw:
			block.append(CNC.gline(x=xmin,y=ymax-r))
			if r>0 :
				block.append(CNC.garc(2,x=xmin+r,y=ymax,i=r,j=0)) 
			if (xmax-xmin)>2*r:
				block.append(CNC.gline(x=xmax-r,y=ymax))
			if r>0 :
				block.append(CNC.garc(2,x=xmax,y=ymax-r,i=0,j=-r))
			if (ymax-ymin)>2*r :
				block.append(CNC.gline(x=xmax,y=ymin+r))
			if r>0 :
				block.append(CNC.garc(2,x=xmax-r,y=ymin,i=-r,j=0))
			if (xmax-xmin)>2*r:
				block.append(CNC.gline(x=xmin+r,y=ymin))
			if r>0 :
				block.append(CNC.garc(2,x=xmin,y=ymin+r,i=0,j=r))
		else : 
			if r>0 :
				block.append(CNC.garc(3,x=xmin+r,y=ymin,i=r,j=0))
			if (xmax-xmin)>2*r:
				block.append(CNC.gline(x=xmax-r,y=ymin))
			if r>0 :
				block.append(CNC.garc(3,x=xmax,y=ymin+r,i=0,j=r))
			if (ymax-ymin)>2*r :
				block.append(CNC.gline(x=xmax,y=ymax-r))
			if r>0 :
				block.append(CNC.garc(3,x=xmax-r,y=ymax,i=-r,j=0))
			if (xmax-xmin)>2*r:
				block.append(CNC.gline(x=xmin+r,y=ymax))
			if r>0 :
				block.append(CNC.garc(3,x=xmin,y=ymax-r,i=0,j=-r))
			if (ymax-ymin)>2*r :
				block.append(CNC.gline(x=xmin,y=ymin+r))
		block.append("(exiting)")
		block.append(CNC.grapid(z=CNC.vars["safe"]))
		blocks.append(block)
		return blocks


#==============================================================================
# Create a simple Rectangle
#==============================================================================
class Tool(Plugin):
	__doc__ = _("Generate a simple rectangle")

	def __init__(self, master):
		Plugin.__init__(self, master, "SimpleRectangle")
		self.icon  = "SimpleRectangle"
		self.group = "Generator"
		self.variables = [
			("xstart",        "float",    0, _("xStart")),
			("xend",        "float",    20, _("xEnd")),
			("ystart",        "float",    0, _("yStart")),
			("yend",        "float",    40, _("yEnd")),
			("radius",        "float",    0, _("Corner Radius")),
			("cw",        "bool",    True, _("clockwise")),
		]
		self.buttons.append("exe")

	# ----------------------------------------------------------------------
	def execute(self, app):
		n = self["name"]
		if not n or n=="default":
			n="SimpleRectangle"
		simpleRectangle = SimpleRectangle(n)
		blocks = simpleRectangle.calc(self["xstart"], self["ystart"], self["xend"], self["yend"], self["radius"],self["cw"])
		active = app.activeBlock()
		if active==0:
			active=1
		app.gcode.insBlocks(active, blocks, _("Create Simple Rectangle"))
		app.refresh()
		app.setStatus(_("Generated: Simple Rectangle"))

if __name__=="__main__":
	simpleRectangle = SimpleRectangle()
	simpleRectangle.calc(10,10,20,30,5,True)
	simpleRectangle.calc(10,10,20,30,50,False)
